.include <mailbox.sci>
.include <fileutil>

main
{
	table identity get(ARGV 1)
	if not(exists(get(identity mbox))) error "$mbox doesn\'t exist" endif
	def mbox = get(identity mbox);
	def newlen = filesize($mbox) lastlen = get(identity lastlen);
	def curpos nextpos;
	if less(int $newlen $lastlen)
		set lastlen = 0
	elsif eq(int $newlen $lastlen)
		echo "nothing to do"
		exit
	endif
	if not(dotlock-wait($mbox 5)) error "cannot lock $mbox" endif
	put identity lastlen $newlen
	onerror error-handler
	defautoseq tolower ascii:Tolower
	defautoseq koi2win koi8r:Tounicode|cp1251:Tonative
	defautoseq win2koi cp1251:Tounicode|koi8r:Tonative
	defautoseq lf linefilter
	defstream mbox file://$mbox basic;
	export stream mbox mbox
	setmailbox(mbox)
	subload base64 base64enc
	export subint base64 base64
	set curpos = $lastlen
	def subject from myid = get(identity id) id passcode
		myemail = get(identity email) mypasscode = get(identity passcode);
	while $curpos
		echo "processing $curpos"
		set nextpos = nextmail($curpos)
		set from = get-email(headerfld($curpos from))
		set subject = trim(headerfld($curpos subject))
		split($subject ':' '' id passcode)
		if eq($id $myid)
			if neqi($from $myemail)
				notify-warning("attempt to access $id from $from")
			elsif neq($passcode $mypasscode)
				notify-warning("wrong passcode, msg headerfld($curpos message-id)")
			else
				process-message($curpos $nextpos)
			endif
		endif
		set curpos = $nextpos
	loop
	rmfile $mbox\.lock
}

process-message(pos limit)
{
	def type = headerfld($pos content-type)
		dispos = headerfld($pos content-disposition)
		fname
		majortype minortype;
	set type = split($type '/' ';' majortype minortype)
	set fname = split($dispos ';' '' dispos)
	echo "$majortype/$minortype"
	if eqi($majortype multipart)
		process-mixed($pos $minortype $type $limit)
		return
	endif
	if eqi($dispos attachment)
		set fname = gethdrparam($dispos filename)
		if not($fname)
			set fname = gethdrparam($type name)
		endif
	else
		set fname = ''
	endif
	if $fname
		process-attachement($pos $majortype $fname $limit)
	elsif eqi($majortype text)
		process-news($pos $type $limit)
	endif
}

process-mixed(pos minortype rest limit)
{
	def nextpos eof;
	def boundary = gethdrparam($rest boundary);
	echo "boundary=$boundary"
	set eof = endofpart(mailbody($pos) $boundary)
	set pos = nextpart($eof $boundary)
	if eqi($minortype alternative)
		process-message($pos $eof)
	else
		while $pos;
			set eof = endofpart($pos $boundary)
			set nextpos = nextpart($eof $boundary)
			process-message($pos $eof)
			set pos = $nextpos
		loop;
	endif
}

process-attachement(pos type fname limit)
{
	set fname = dosbasename(basename($fname))
	def dir;
	set type = translate(reset(tolower) $type)
	if check(identity $type)
		set dir = get(identity $type)
	elsif check(identity *)
		set dir = get(identity *)
	else
		notify-warning("unknown MIME type $type for $fname")
		return
	endif
	open-encoded-stream(instream $fname $pos)
	defstream outstream file://$dir/$fname basic true;
	copy-stream(instream outstream $limit)
	release stream instream
	release stream outstream
}

process-news(pos params limit)
{
	if not(check(identity news)) return endif
	def newsfile  = get(identity news)
		newsext  =  get(identity newsext)
		newscharset = get(identity newscharset)
		newsmax     = get(identity newsmax) tmp pos1;
	open-encoded-stream(instream news $pos)
	set tmp = readfrom(instream lf)
	if isprefix(trim($tmp) --process)
		switch $tmp
			when set(pos1 index($tmp delete))
				set tmp = substr($tmp add($pos1 length(delete)))
				rmfile $(newsfile)\.trim($tmp)$newsext
				breaksw
			when set(pos1 index($tmp replace))
				set tmp = substr($tmp add($pos1 length(delete)))
				defstream outstream file://$(newsfile)\.trim($tmp)$newsext basic true
				copy-stream-trn(instream outstream $limit detect-translation(gethdrparam($params charset) $newscharset))
				release stream outstream
				breaksw
			when true
				shift-news-files($newsfile $newsext $newsmax)
				defstream outstream file://$newsfile\.0$newsext basic true
				copy-stream-trn(instream outstream $limit detect-translation(gethdrparam($params charset) $newscharset))
				release stream outstream
		endsw
	endif
	release stream instream
}


shift-news-files(newsfile newsext newsmax)
{
	def i = $newsmax;
	while neq(int $i 0)
		if exists($newsfile\.sub($i 1)$newsext)
			mvfile $newsfile\.sub($i 1)$newsext $newsfile\.$i$newsext
		endif
		decr i 1
	loop
	rmfile $newsfile\.$newsmax$newsext
}

detect-translation(src-charset dest-charset)
{
	switch $src-charset
		when eqi($^ koi8r)
		when eqi($^ koi8-r)
		when eqi($^ koi8)
			return(cond(eq($dest-charset cp1251) koi2win))
		when eqi($^ windows-1251)
		when eqi($^ cp1251)
		when eqi($^ cp-1251)
			return(cond(eq($dest-charset koi8r) win2koi))
	endsw	
}

open-encoded-stream(name fname pos)
{
	def encoding = trim(headerfld($pos content-transfer-encoding));
	switch $encoding
		when eqi($^ 7bit)
		when eqi($^ 8bit)
		when eqi($^ binary)
		when not($^)
			defstream $name 'dup://mbox' basic;
			breaksw
		when eqi($^ base64)
			defstream $name 'script[base64]://dup://mbox' basic;
			breaksw
		when true
			notify-warning("unsupported encoding $encoding for $fname")
			return
	endsw
	setpos instream mailbody($pos)
}

copy-stream(in out limit)
{
	while;
		if eq(int getpos($in) $limit) break endif
		readfrom($in)
		if iseof() break endif
		writetagto $out;
	loop;
}

copy-stream-trn(in out limit au)
{
	def str;
	while;
		if eq(int getpos($in) $limit) break endif
		set str = readfrom($in $au)
		if iseof() break endif
		if $str writeto $out $str endif
		if $[] writetagto $out; endif
	loop;
}
get-email(str)
{
	def idx = index($str \<) email;
	if $idx
		split(substr($str add($idx 1)) '>' '' email)
		return($email)
	else
		return(trim($str))
	endif
}

notify-error(msg)
{
	defstream mailer "pipe:///usr/sbin/sendmail get(identity email)" basic true;
	writeto mailer "To: get(identity email)\n"
	writeto mailer "Subject: fatal job processing error\n"
	writeto mailer '\n'
	writeto mailer $msg\n
	rmstream mailer
	error "$msg"
}

notify-warning(msg)
{
	defstream mailer "pipe:///usr/sbin/sendmail get(identity email)" basic true;
	writeto mailer "To: get(identity email)\n"
	writeto mailer "Subject: job processing failure\n"
	writeto mailer '\n'
	writeto mailer $msg\n
	echo "$msg"
	rmstream mailer
}

error-handler(desc)
{
	rmfile get(identity mbox).lock
	echo $desc
	exit
}
